/*
    This source code contains routines that will strip copier/backup
    unit headers from GameBoy ROM images so they will work properly
    with GameBoy emulators.

    BorlandC 3.1 was used to compile this successfully.

    It is a crime to redistribute these routines in a commercial
    venture of any kind without permission or licensing agreement.
    Contact us at our web page for more information on licensing.

    (C)opyright 1994, 1995, 1996 Damaged Cybernetics

    If you have any questions or comments, please contact the
    following ppl via the Internet:

        Donald Moore   (MindRape) mindrape@goodnet.com
        Donald Staheli (Royce)     staheli@goodnet.com

        Web Page: www.futureone.com/~damaged

 */
#include <stdio.h>
#include <stdlib.h>
#include "gameboy.h"


/* Title/(C)opyright msg */
char szTitleMsg[] = {
   "GBStrip - V1.5 - GB Header Stripper for Virtual Gameboy\n"
   "(C) 1994, 1995, 1996 Damaged Cybernetics (www.futureone.com/~damaged)\n"
};

/* generic help msg */
char szHelpMsg[] = {
  "Usage: GBStrip <source.gb> <dest.gb>\n\n"
  "GBStrips strips headers from GameBoy ROM images so they will work\n"
  "with various GameBoy emulators that are available.\n"
};


void help(void);
void GBStrip(char *origfilename,char *destfilename);

/*
   void help(void)
   ENT: <nothing>
   RET: <nothing>

   display a help message to the user
 */
void help()
{
   puts(szHelpMsg);
   exit(0);
}


/*
    void GBStrip(char *origfilename,char *destfilename)
    ENT: char *cOrigFName - GameBoy file to read from
         char *cDestFName - New GameBoy image to write
    RET: <nothing>

    Attempt to find a header on the GameBoy ROM image, if
    one exists remove the first 512 bytes (which is a
    standard for most copiers).

    Headers on ROM images help copiers to determine what
    kind of ROM banking emulation, external hardware is
    required and built in deprotection schemes.

    This function now uses the new GameBoy.c library.
 */
void GBStrip(char *cOrigFName,char *cDestFName)
{ FILE *source;
  FILE *dest;
  char cBuffer[BUFFER_SIZE];
  unsigned int i;

  if ((source = fopen(cOrigFName,"rb")) == NULL)
    {
      printf("Error: Cannot open input file '%s'\n",cOrigFName);
      exit(1);
    };

  /* determine if the ROM image supplied is Valid and has a header */
  switch (DetectGameBoy(source))
    {
      case DETECT_GAMEBOY_RAW    : puts("GameBoy ROM image does not have a header!\nAborting...");
                                   fclose(source);
                                   exit(1);

      case DETECT_GAMEBOY_INVALID: puts("Not a valid GameBoy ROM image!\nAborting...");
                                   fclose(source);
                                   exit(1);
    }

  puts("Detected GameBoy ROM image with a header.");

  if ((dest = fopen(cDestFName,"wb")) == NULL)
    {
      printf("Error: Cannot create output file '%s'\n",cDestFName);
      exit(1);
    };


  /* skip HEADER_SIZE # of bytes, GameBoy emulators don't need this */
  fseek(source,HEADER_SIZE,SEEK_SET);

  /* now copy the rest of the bin to the new destination */
  while (!feof(source))
    {
       i = fread(cBuffer,1,BUFFER_SIZE,source);
       fwrite(cBuffer,1,i,dest);
       printf("o");
    }

  puts("\nFinished!");
  fcloseall();
}


void main(int argc,char *argv[])
{
  puts(szTitleMsg);    /* throw out our title msg */

  if (argc != 3)
    help();

  GBStrip(argv[1],argv[2]);
}
